-----------------------------------------------------------------------------------------------
-- Client Lua Script for EzMessenger
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
require "Window"
 
-----------------------------------------------------------------------------------------------
-- EzMessenger Module Definition
-----------------------------------------------------------------------------------------------
local EzMessenger = {} 

local function CPrint(string)
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_Command, string, "")
end

-- table dump functions
function table.val_to_str ( v )
  if "string" == type( v ) then
    v = string.gsub( v, "\n", "\\n" )
    if string.match( string.gsub(v,"[^'\"]",""), '^"+$' ) then
      return "'" .. v .. "'"
    end
    return '"' .. string.gsub(v,'"', '\\"' ) .. '"'
  else
    return "table" == type( v ) and table.tostring( v ) or
      tostring( v )
  end
end
function table.key_to_str ( k )
  if "string" == type( k ) and string.match( k, "^[_%a][_%a%d]*$" ) then
    return k
  else
    return "[" .. table.val_to_str( k ) .. "]"
  end
end
function table.tostring( tbl )
  local result, done = {}, {}
  for k, v in ipairs( tbl ) do
    table.insert( result, table.val_to_str( v ) )
    done[ k ] = true
  end
  for k, v in pairs( tbl ) do
    if not done[ k ] then
      table.insert( result,
        table.key_to_str( k ) .. "=" .. table.val_to_str( v ) )
    end
  end
  return "{" .. table.concat( result, "," ) .. "}"
end
function table.reverse ( tab )
    local size = #tab
    local newTable = {}
 
    for i,v in ipairs ( tab ) do
        newTable[size-i] = v
    end
 
    return newTable
end

function deepcopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[deepcopy(orig_key)] = deepcopy(orig_value)
        end
        setmetatable(copy, deepcopy(getmetatable(orig)))
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end
local ChatLog = {
	Player = "", -- players name
	Type = "",
	ChatLog = { },-- amalgamation of all message
	Form = nil, -- an object	
	Pending = 0,
} 

local DefaultSettings = {
	messengerFramePosition = {
		left = 100,
		top = 100,
		bottom = 450,
		right = 700,		
	},
	
	minBtnFramePosition  = {
		left = 100,
		top = 100,
		bottom = 142,
		right = 180,		
	},
	
	bEnableMinimisedState = true,
	bPopupOnMessage = true,
	bSurpressInCombat = true,
	bMinimisedState = false,
	bShowTimestamp = true,
	bShowSizingFrame = false,
	bKeepFocus = false,
	
	cMeColour = "xkcdGold",
	cWhisperColour = "ChannelWhisper",
	cAccountWhisperColor = "ChannelAccountWisper",
}


-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function EzMessenger:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

	self.settings = deepcopy(DefaultSettings)
	self.data = { 
		unseenConversationCount = 0,
		activeChatWindow = "",
		bNewConvoWindowActive = false,
		bQuickIMActive = false,
	}

    return o
end

function EzMessenger:Init()	
    Apollo.RegisterAddon(self, false, "",  { })

end
 

-----------------------------------------------------------------------------------------------
-- EzMessenger OnLoad
-----------------------------------------------------------------------------------------------
function EzMessenger:OnLoad()
    -- load our form file
	Apollo.RegisterSlashCommand("ezim", "OnEzMessengerSlash", self)
	self.xmlDoc = XmlDoc.CreateFromFile("EzMessenger.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

function EzMessenger:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return nil
	else
		return self.settings
	end	
end

function EzMessenger:OnRestore(eType, tData)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then 
		return 
	end		

	--Load in settings	
	for Dkey, Dvalue in pairs(self.settings) do
		if tData[Dkey] ~= nil then
			self.settings[Dkey] = tData[Dkey]
		end
	end	
end


function EzMessenger:ApplySettings()	
	-- positions
	self.messageManagerContainer:SetAnchorOffsets(
		self.settings.messengerFramePosition.left,
	 	self.settings.messengerFramePosition.top,
		self.settings.messengerFramePosition.right,
	 	self.settings.messengerFramePosition.bottom)
	
	self.messageManagerMinimised:SetAnchorOffsets(
		self.settings.minBtnFramePosition.left,
	 	self.settings.minBtnFramePosition.top,
		self.settings.minBtnFramePosition.right,
	 	self.settings.minBtnFramePosition.bottom)
	
	-- minimise and maximise accordingly
	if self.settings.bMinimisedState then
		self:OnMessengerMin()
	else
		self:OnMessengerMaximise()
	end
	
	--
	self.messageManagerForms.SizeingFrame:Show(self.settings.bShowSizingFrame)
end

-----------------------------------------------------------------------------------------------
-- EzMessenger OnDocLoaded
-----------------------------------------------------------------------------------------------
function EzMessenger:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	
		self.messageManagerContainer = Apollo.LoadForm(self.xmlDoc, "EzMessageManager", nil, self)
		self.messageManagerMinimised = Apollo.LoadForm(self.xmlDoc, "EzMessengerMinimised", nil, self)
		self.messageManagerMinimised:SetData(0)
		self.messageManagerMinimised:FindChild("Pending"):SetText(0)

		local c = self.messageManagerContainer:FindChild("ActiveConvo")
		self.messageManagerForms = {
			ConversationList = self.messageManagerContainer:FindChild("List"),
			ActiveChatLog = c,	
			NewConvoWindow = Apollo.LoadForm(self.xmlDoc, "NewConvoPopup", c, self),
			SizeingFrame = self.messageManagerContainer:FindChild("MovableHandle"),	
			QuickIM = Apollo.LoadForm(self.xmlDoc, "QuickIM", c, self),
		}
		
		self.messageManagerForms.QuickIM:Show(false)
		self.messageManagerForms.NewConvoWindow:Show(false)
		self:ApplySettings()
				
		self.optionsWindow = Apollo.LoadForm(self.xmlDoc, "EzMessengerSettings", nil, self)
		self.optionsWindow:Show(false)
		
		Apollo.RegisterEventHandler("ChatMessage", "OnChatMessage", self)
		Apollo.RegisterEventHandler("ChatTellFailed", "OnChatTellFailed", self)
		Apollo.RegisterEventHandler("ChatAccountTellFailed",	"OnChatTellFailed", self)
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListLoaded", self)
		Apollo.RegisterEventHandler("EzMessengerMaximise", "ToggleWindow", self)
		--Apollo.RegisterEventHandler("ItemLink", "OnItemLink", self)
	end
end


function EzMessenger:OnInterfaceMenuListLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", 
		"EzMessenger", {"EzMessengerMaximise", "", "IconSprites:Icon_Windows_UI_CRB_Questlog_Call"})
end

function EzMessenger:UpdateInterfaceMenuAlerts()
	local nUnseen = self.data.unseenConversationCount
	Event_FireGenericEvent("InterfaceMenuList_AlertAddOn", "EzMessenger", {nUnseen > 0, nil, nUnseen })

end

--toggles the Window on or Off
function EzMessenger:ToggleWindow()
	if self.settings.bMinimisedState then
		self:OnMessengerMaximise()
	else
		self:OnMessengerMin()
	end
end

-----------------------------------------------------------------------------------------------
-- EzMessenger Functions
-----------------------------------------------------------------------------------------------
function EzMessenger:OnEzMessengerSlash(sCmd, sInput)
	local s = string.lower(sInput)	
	if s == "options" then
		self:ShowOptionsMenu()
	elseif s == "reset" then
		self.settings = deepcopy(DefaultSettings)
		self:ApplySettings()
		
	elseif s == "spamme" then	
		for i=0, 25 do
			self:OnNewMessage("Tester", false, string.format("Test Message %i", i), "Whisper")
		end
	end	
end

function EzMessenger:OnChatTellFailed(channel, strCharacterTo)
	CPrint("Tell to " .. strCharacterTo .. " failed")
end

function EzMessenger:OnChatMessage(channelCurrent, tMessage)
	local tChannelType = channelCurrent:GetType()	
	
	if tChannelType == ChatSystemLib.ChatChannel_Whisper then
		local sender = tMessage.strSender
		local realm = tMessage.strRealmName
		local bself = tMessage.bSelf
		local msg = tMessage.arMessageSegments[1].strText		
		self:OnNewMessage(sender, bself, msg, "Whisper")
	
	elseif tChannelType == ChatSystemLib.ChatChannel_AccountWhisper then
		local sender = tMessage.strSender
		local realm = tMessage.strRealmName
		local msg = tMessage.arMessageSegments[1].strText
		local bself = tMessage.bSelf
	
		if realm ~= "" and realm ~= nil then
			sender = sender .. "@" .. realm
		end	
		self:OnNewMessage(sender, bself , msg, "AccountWhisper")	
	end
end

function EzMessenger:OnNewMessage(sender, bself , message, type)	
	-- if i do not have a convo with [player] create one
	local conversationTab = self:GetConversationTabFor(sender)
	
	if conversationTab == nil then
		self:RegisterNewConversation(sender, type)
		conversationTab = self:GetConversationTabFor(sender)
		self.data.activeChatWindow = sender	
	end	
	
	local colour = ""
	if bself then
		sender = "Me"
		colour = self.settings.cMeColour
	else
		if type == "Whisper" then
			colour = self.settings.cWhisperColour
		else
			colour = self.settings.cAccountWhisperColor
		end	
	end

	-- add this message to the players conversation	 
	local entry = self:BuildNewChatEntry(sender, message, colour, conversationTab )	
	table.insert(conversationTab:GetData().ChatLog, entry) 	
	
	self:AddMessagePendingFor(sender)
end

------------------------------------------------------
--  UI Building Functions
------------------------------------------------------
-- BuildNewConversationTab() :: Creates a new Conversation from scratch
function EzMessenger:RegisterNewConversation(data, type)
	--new conversation, add a button to the convo list
	local newBtn = self:BuildNewConversationTab(data)
	
	-- create a chat log object, and attach it to the child.
	newBtn:SetData(deepcopy(ChatLog))
	newBtn:GetData().Player = data
	newBtn:GetData().Form = self:BuildNewConvoForm(data)
	newBtn:GetData().Type = type
	newBtn:GetData().Pending = 0
end

-- BuildNewConversationTab() :: Creates and Adds a new Conversation tab to the UI
function EzMessenger:BuildNewConversationTab(player)
	--player is given as either velken or velken@Ascendancy
	local newConvoIcon = Apollo.LoadForm(self.xmlDoc, "ConversationButton", self.messageManagerForms.ConversationList, self)
	newConvoIcon:SetText(player)
	newConvoIcon:SetName(player)
	self.messageManagerForms.ConversationList:ArrangeChildrenVert()
	return newConvoIcon	
end

-- BuildNewConvoForm() :: Creates a new Conversation Form
function EzMessenger:BuildNewConvoForm(player)
	--player is given as either velken or velken@Ascendancy
	local newConvoForm = Apollo.LoadForm(self.xmlDoc, "ChatLogForm", self.messageManagerForms.ActiveChatLog, self)	
	newConvoForm:SetName(player)
	newConvoForm:Show(false)		
	newConvoForm:FindChild("Header"):SetText(player)
	
	return newConvoForm	
end


-- BuildNewChatEntry() :: Creates a new Chat lines object for the Conversation Form
function EzMessenger:BuildNewChatEntry(player, message, colour, tab)
	local newWnd = Apollo.LoadForm(self.xmlDoc, "ChatLine", tab:GetData().Form:FindChild("History"), self)
	
	local tm = GameLib.GetLocalTime()
	local timeStamp = string.format("%d:%02d", tm.nHour, tm.nMinute)
	
	newWnd:SetTextColor(ApolloColor.new(colour))
	if self.settings.bShowTimestamp then	
		newWnd:SetText(string.format("[%s] %s: %s", timeStamp, player, message))
	else
		newWnd:SetText(string.format("%s: %s", player, message))
	end	
	
	newWnd:SetHeightToContentHeight()
	
	local history = tab:GetData().Form:FindChild("History")	
	history:ArrangeChildrenVert()	
	history :SetVScrollPos(history:GetVScrollRange())
	
	return newWnd
end

function EzMessenger:BuildFriendButton(player, isAccount)
	local newIcon = Apollo.LoadForm(self.xmlDoc, "FriendsButton", 
		self.messageManagerForms.QuickIM:FindChild("FriendsList"), self)
	
	newIcon:FindChild("Account"):Show(isAccount)
	newIcon:SetText(player)	
	self.messageManagerForms.QuickIM:FindChild("FriendsList"):ArrangeChildrenVert()
	return newIcon
end

function EzMessenger:BuildGuildMemberButton(player)
	local newIcon = Apollo.LoadForm(self.xmlDoc, "FriendsButton", 
		self.messageManagerForms.QuickIM:FindChild("GuildList"), self)
	
	newIcon:SetText(player)	
	self.messageManagerForms.QuickIM:FindChild("GuildList"):ArrangeChildrenVert()
	return newIcon
end


-----------------------------------------
-- Helper Functions
-----------------------------------------
-- OnMessageReceivedFinished() :: Called once a message has been received
function EzMessenger:OnMessageReceivedFinished(sender)
	self:AddMessagePendingFor(sender)
	if self.settings.bMinimisedState and self.settings.bPopupOnMessage then
		
		local isInCombat = GameLib.GetPlayerUnit():IsInCombat()
		if isInCombat and self.settings.bSurpressInCombat then
			return
		end
	
		self:OnMessengerMaximise()
		self:SetActiveChatWindow(sender)
	end
end

-- AddMessagePendingFor() :: Increments Messages Pending for specified player
function EzMessenger:AddMessagePendingFor(sender)
			
	local isActiveConvo = sender == self.data.activeChatWindow
	local isMe = sender == "Me"
	
	--CPrint(string.format("Message Pending from '%s' p={bMin=%s me=%s active=%s}  ", 
		--sender, 
		--tostring(self.settings.bMinimisedState),
		--tostring(isMe),
		--tostring(isActiveConvo)))
		
	if isMe then return end -- i sent the message, i dont want to know	
	self.data.unseenConversationCount = self.data.unseenConversationCount + 1 -- one more unseen
		
	-- Add a message unseen to the Minimised Icon
	-- Update the Minimised Icon	
	local minPendingWnd = self.messageManagerMinimised:FindChild("Pending")
	minPendingWnd:SetText(self.data.unseenConversationCount)	
	
	-- Update the senders button
	if not isActiveConvo then
		local tab = self:GetConversationTabFor(sender)
		local tData = tab:GetData()
		tData.Pending = tData.Pending + 1	
		tab:FindChild("Pending"):SetText(tData.Pending)
	end
	
	self:UpdateInterfaceMenuAlerts()
end
-- ClearPendingMessagesFor() :: Clears pending Messages for specified player
function EzMessenger:ClearPendingMessagesFor(sender)
	--CPrint("Clearing messages for " .. sender)
	local pending = 0
	
	if sender ~= "" then	
		local tab = self:GetConversationTabFor(sender)		
		local data = tab:GetData()
		pending = data.Pending	
		data.Pending = 0
		tab:FindChild("Pending"):SetText("")	
	end
	
	local minPendingWnd = self.messageManagerMinimised:FindChild("Pending")
	self.data.unseenConversationCount = math.max(0, self.data.unseenConversationCount - pending)
	minPendingWnd:FindChild("Pending"):SetText(self.data.unseenConversationCount)
	
	self:UpdateInterfaceMenuAlerts()	
end

-- GetConversationTabFor() :: returns the conversation tab for the player, nil if not found
function EzMessenger:GetConversationTabFor(player)
	return self.messageManagerForms.ConversationList:FindChild(player)
end

-- SetActiveChatWindow() :: Sets the active Chat Window to 'Player'
function EzMessenger:SetActiveChatWindow(player)
	self:CloseActiveChatWindow()
	self.data.activeChatWindow = player
	self.messageManagerForms.ActiveChatLog:FindChild(player):Show(true)
end
-- CloseActiveChatWindow() :: Closes the Active Chat Window
function EzMessenger:CloseActiveChatWindow()
	if self.data.activeChatWindow == "" then
		return			
	end
		
	self.messageManagerForms.ActiveChatLog:FindChild(self.data.activeChatWindow):Show(false)	
	self.data.activeChatWindow = ""
end

-- SetStartConversationWindow() :: state = true or flase, 
function EzMessenger:SetStartConversationWindow(state)
	if state then
		self:CloseActiveChatWindow()
		self.messageManagerContainer:FindChild("NewConvoBtn"):SetText("-")
		self.messageManagerContainer:FindChild("NewConvoBtn"):SetBGColor(ApolloColor.new("white"))
	else
		self.messageManagerContainer:FindChild("NewConvoBtn"):SetText("+")
		self.messageManagerContainer:FindChild("NewConvoBtn"):SetBGColor(ApolloColor.new("green"))
	end

	self.messageManagerForms.NewConvoWindow:Show(state)
	self.data.bNewConvoWindowActive = state 
end
function EzMessenger:SetQuickIMState(state)
	if state then
		self:CloseActiveChatWindow()		
	end

	self.messageManagerForms.QuickIM:Show(state)
	self.data.bQuickIMActive = state 
end
-----------------------------------------------------------------------------------------------
-- EzMessengerForm Functions
-----------------------------------------------------------------------------------------------
function EzMessenger:OnMessengerClose(wndHandler, wndControl, eMouseButton )
	self.messageManagerContainer:Show(false)
end
function EzMessenger:OnMessengerMin(wndHandler, wndControl, eMouseButton )
	self.settings.bMinimisedState = true	
	self.messageManagerContainer:Show(false)
	
	self:ClearPendingMessagesFor("")
	
	if self.settings.bEnableMinimisedState then
		self.messageManagerMinimised:Show(true)
	end
end
function EzMessenger:OnMessengerMaximise(wndHandler, wndControl, eMouseButton )
	self:ClearPendingMessagesFor(self.data.activeChatWindow)
	self.settings.bMinimisedState = false
	self.messageManagerContainer:Show(true)
	self.messageManagerMinimised:Show(false)
end
function EzMessenger:OnConversationIconClicked(wndHandler, wndControl, eMouseButton )

	local senderClicked = wndControl:GetText()
	self:CloseActiveChatWindow()		
	self:SetStartConversationWindow(false)
	self:SetActiveChatWindow(senderClicked)
	self:ClearPendingMessagesFor(senderClicked)
		
	-- set the scroll to max
	local form = wndControl:GetData().Form
	form:FindChild("History"):SetVScrollPos(form:FindChild("History"):GetVScrollRange())
end
function EzMessenger:OnQuickIMInflate( wndHandler, wndControl, eMouseButton )
	
	self.messageManagerForms.QuickIM:FindChild("FriendsList"):DestroyChildren()
	self.messageManagerForms.QuickIM:FindChild("GuildList"):DestroyChildren()
	
	-- Add Friends
	for key, tAccFriend in pairs(FriendshipLib.GetAccountList()) do
		if tAccFriend.arCharacters then 
			self:BuildFriendButton(tAccFriend.strCharacterName, true)
		end
	end	
	for key, tFriend in pairs(FriendshipLib.GetList()) do
		if tFriend.fLastOnline == 0 then
			self:BuildFriendButton(tFriend.strCharacterName, false)
		end			
	end	
		
	--show the Friends Window
	self:SetQuickIMState(not self.data.bQuickIMActive)
end

---------------------------------------------------------------------------------------------------
-- EzMessageManager Functions
---------------------------------------------------------------------------------------------------
function EzMessenger:OnMessengerWindowMoved( wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom)	
	local l,t,r,b = wndControl:GetAnchorOffsets()
	self.settings.messengerFramePosition = {
		left = l,
		top = t,
		right = r,
		bottom = b,				
	}
end

function EzMessenger:OnMessengerMinWindowMoved( wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom)
	local l,t,r,b = wndControl:GetAnchorOffsets()
	self.settings.minBtnFramePosition= {
		left = l,
		top = t,
		right = r,	
		bottom = b,			
	}
end
function EzMessenger:OnMessengerSettings( wndHandler, wndControl, eMouseButton )
	self:ShowOptionsMenu()
end

function EzMessenger:OnShowNewConvoWindow( wndHandler, wndControl, eMouseButton )
	--show the new conversation window
	if self.data.bNewConvoWindowActive then		
		self:SetStartConversationWindow(false)
	else
		self:CloseActiveChatWindow()
		self:SetStartConversationWindow(true)
	end
end

function EzMessenger:OnNewConversationClose( wndHandler, wndControl, eMouseButton )
	-- show the current convo window (if there is any)
	self:SetStartConversationWindow(false)
end

function EzMessenger:OnChatSend(wndHandler, wndControl)
	self:OnSendButton(wndHandler, wndControl, nil)
end
function EzMessenger:OnSendButton(wndHandler, wndControl, eMouseButton )
	-- must be the active forms send button that was pressed
	local input = wndControl:GetParent():FindChild("Message")
	local msg = input:GetText()
	local player = self.data.activeChatWindow
	
	local tab = self.messageManagerForms.ConversationList:FindChild(player)	
	local data = tab:GetData()
	
	--msg = self:HelperReplaceLinks(msg, wndControl:GetParent():FindChild("Message"):GetAllLinks())
	
	local str = ""
	if data.Type == "Whisper" then
		str = string.format("/w %s %s", player, msg)
	else
		str = string.format("/aw %s %s", player, msg)
	end
	--CPrint("Sending: '" .. str .. "'")
	ChatSystemLib.Command(str)		
	input:SetText("")
	
	if self.settings.bKeepFocus then
		input:SetFocus()
	end
end

function EzMessenger:OnCreateConvoSend(wndHandler, wndControl, eMouseButton )
	-- New Conversation created
	local parent = wndControl:GetParent()
	local target = parent:FindChild("Receiver"):GetText()
	local message =  parent:FindChild("Message"):GetText()
	
	ChatSystemLib.Command(string.format("/w %s %s", target , message))
	parent:FindChild("Receiver"):SetText("")
	parent:FindChild("Message"):SetText("")
	
	self.data.activeChatWindow = target -- not sure here, what if /w fails?
	self:OnNewConversationClose()			
end

function EzMessenger:OnMessengerWindowEscape( wndHandler, wndControl )
	self:ToggleWindow()
end
---------------------------------------------------------------------------------------------------
-- EzMessengerSettings Functions
---------------------------------------------------------------------------------------------------
--Check boxes
function EzMessenger:OnIncMessageCheck(wndHandler, wndControl, eMouseButton )
	self.settings.bPopupOnMessage = wndControl:IsChecked()
end
function EzMessenger:OnSurpressCheck(wndHandler, wndControl, eMouseButton )
	self.settings.bSurpressInCombat = wndControl:IsChecked()
end
function EzMessenger:OnTimestampCheck(wndHandler, wndControl, eMouseButton )
	self.settings.bShowTimestamp = wndControl:IsChecked()
end
function EzMessenger:OnEnableMinCheck( wndHandler, wndControl, eMouseButton )
	self.settings.bEnableMinimisedState = wndControl:IsChecked()
end
function EzMessenger:OnSizeFrameCheck( wndHandler, wndControl, eMouseButton )
	self.settings.bShowSizingFrame = wndControl:IsChecked()
	self:ApplySettings()
end
function EzMessenger:OnFocusKeepCheck( wndHandler, wndControl, eMouseButton )
	self.settings.bKeepFocus = wndControl:IsChecked()
end
-- colour changers
function EzMessenger:ResponseColourChanged(wndHandler, wndControl, strText )
	self.settings.cMeColour = strText
	self.optionsWindow:FindChild("ResponseColourInput"):SetTextColor(ApolloColor.new(self.settings.cMeColour))
end
function EzMessenger:OnWhisperColourChanged(wndHandler, wndControl, strText )
	self.settings.cWhisperColour= strText
	self.optionsWindow:FindChild("WhisperColourInput"):SetTextColor(ApolloColor.new(self.settings.cWhisperColour))
end
function EzMessenger:OnAccountColourChanged( wndHandler, wndControl, strText )
	self.settings.cAccountWhisperColor= strText
	self.optionsWindow:FindChild("AccountWhisperColourInput"):SetTextColor(ApolloColor.new(self.settings.cAccountWhisperColor))
end

--Show and Hide
function EzMessenger:OnOptionsMenuClose(wndHandler, wndControl, eMouseButton )
	self.optionsWindow:Show(false)
end
function EzMessenger:ShowOptionsMenu()	
	self.optionsWindow:FindChild("btnOpenOnInc"):SetCheck(self.settings.bPopupOnMessage)
	self.optionsWindow:FindChild("btnSurpress"):SetCheck(self.settings.bSurpressInCombat)
	self.optionsWindow:FindChild("btnShowTimestamp"):SetCheck(self.settings.bShowTimestamp)
	self.optionsWindow:FindChild("btnEnableMinimisedIcon"):SetCheck(self.settings.bEnableMinimisedState )
	self.optionsWindow:FindChild("btnShowSizeableFrame"):SetCheck(self.settings.bShowSizingFrame )
	self.optionsWindow:FindChild("btnKeepFocus"):SetCheck(self.settings.bKeepFocus)	
		
	self.optionsWindow:FindChild("ResponseColourInput"):SetText(self.settings.cMeColour)
	self.optionsWindow:FindChild("ResponseColourInput"):SetTextColor(ApolloColor.new(self.settings.cMeColour))
		
	self.optionsWindow:FindChild("WhisperColourInput"):SetText(self.settings.cWhisperColour)
	self.optionsWindow:FindChild("WhisperColourInput"):SetTextColor(ApolloColor.new(self.settings.cWhisperColour))
	
	self.optionsWindow:FindChild("AccountWhisperColourInput"):SetText(self.settings.cAccountWhisperColor)
	self.optionsWindow:FindChild("AccountWhisperColourInput"):SetTextColor(ApolloColor.new(self.settings.cAccountWhisperColor))
	
	self.optionsWindow:Show(true)
end
---------------------------------------------------------------------------------------------------
-- ChatLogForm Functions
---------------------------------------------------------------------------------------------------
function EzMessenger:OnFriendAdd( wndHandler, wndControl, eMouseButton )
	ChatSystemLib.Command("/Friend " .. self.data.activeChatWindow)
end
function EzMessenger:OnAccountFriendAdd( wndHandler, wndControl, eMouseButton )
	ChatSystemLib.Command("/friendAccount " .. self.data.activeChatWindow)
end
function EzMessenger:OnIgnoreAdd( wndHandler, wndControl, eMouseButton )
	ChatSystemLib.Command("/Ignore " .. self.data.activeChatWindow)
end
function EzMessenger:OnGroupInvite( wndHandler, wndControl, eMouseButton )
	ChatSystemLib.Command("/Invite " .. self.data.activeChatWindow)
end

function EzMessenger:OnChatWindowClose( wndHandler, wndControl, eMouseButton )	
	local activeChatIcon = self:GetConversationTabFor(self.data.activeChatWindow)
	local activeChatTab = self.messageManagerForms.ActiveChatLog:FindChild(self.data.activeChatWindow)
	
	activeChatIcon:Destroy()
	activeChatTab:Destroy()	
	
	self.data.activeChatWindow = ""
	self.messageManagerForms.ConversationList:ArrangeChildrenVert()
end
---------------------------------------------------------------------------------------------------
-- FriendsButton Functions
---------------------------------------------------------------------------------------------------

function EzMessenger:OnFriendsButtonClick( wndHandler, wndControl, eMouseButton )
	local friend = wndControl:GetText()
	local isAccount = wndControl:FindChild("Account"):IsShown()
	
	--CPrint(string.format("Opening for Friend '%s' {isaccount=%s}", 
	--	friend, tostring(isAccount)))
		
	if isAccount then
		self:RegisterNewConversation(friend, "Account")
	else
		self:RegisterNewConversation(friend, "Whisper")
	end
	--self.data.activeChatWindow = friend
	self:SetActiveChatWindow(friend)
	self:SetQuickIMState(false)
end

-----------------------------------------------------------------------------------------------
-- EzMessenger Instance
-----------------------------------------------------------------------------------------------
local EzMessengerInst = EzMessenger:new()
EzMessengerInst:Init()
